from typing import Optional, Union, Any, Literal, TypedDict
from pathlib import Path
from .data_container import *
from .flojoy_python import *
from .job_result_builder import *
from .flojoy_instruction import *
from .plotly_utils import *
from .module_scraper import *
from .job_result_utils import *
from .utils import *
from .parameter_types import *
from .small_memory import *
from .flojoy_node_venv import *
from .job_service import *
from .node_init import *
from .node_preflight import *
from .data_container import *
from .config import *
from .flojoy_cloud import *
from .models import *

def flojoy(
    original_function: Callable[..., DataContainer | dict[str, Any] | TypedDict | None]
    | None = None,
    *,
    node_type: Optional[str] = None,
    deps: Optional[dict[str, str]] = None,
    inject_node_metadata: bool = False,
    inject_connection: bool = False,
) -> Callable[..., DataContainer | dict[str, Any] | None]: ...



from threading import Lock

from flojoy.parameter_types import (
    HardwareConnection,
    HardwareDevice,
    CameraDevice,
    CameraConnection,
    SerialDevice,
    SerialConnection,
    VisaDevice,
    VisaConnection,
)
from typing import Any
from .config import logger

_connection_lock = Lock()


class DeviceConnectionManager:
    handles: dict[str | int, HardwareConnection] = {}

    @classmethod
    def register_connection(cls, device: HardwareDevice, connection: Any):
        with _connection_lock:
            id = device.get_id()
            if id in cls.handles:
                raise ValueError(f"Connection with id {id} already exists")

            match device:
                case CameraDevice():
                    cls.handles[id] = CameraConnection(connection)
                case SerialDevice():
                    cls.handles[id] = SerialConnection(connection)
                case VisaDevice():
                    cls.handles[id] = VisaConnection(connection)

    @classmethod
    def get_connection(cls, id: str | int) -> HardwareConnection:
        with _connection_lock:
            return cls.handles[id]

    @classmethod
    def clear(cls):
        with _connection_lock:
            cls.handles.clear()

        logger.debug(f"Connections closed: {cls.handles}")
